#include <iostream>
using namespace std;

#include <Inventor/SoDB.h>
#include <Inventor/Win/SoWin.h>
#include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoFaceSet.h>

const int xmax = 2, ymax = 2;
const int  vertex = (xmax)*(ymax)*4;
const int  faces = (xmax*ymax);
// Positions of all of the vertices:
static float vertexPositions[vertex][3];
static int32_t indices[faces];
// Colors for the 12 faces
static float colors[faces][3];
// Normals for each polygon:
static float norms[faces][3]; 
void creaVertexIndicesColores(){
	cout<<"creaVertexIndicesColores"<<endl;
	//primero crea los vertices
	int zz = 0;
	for(int i=0; i < ymax;++i){
		for(int j=0; j < xmax;++j){

			vertexPositions[zz][0] = float(j);
			vertexPositions[zz][1] = float(i);
			vertexPositions[zz][2] = 0.0f;
			
			vertexPositions[zz+1][0] = float(j+1);
			vertexPositions[zz+1][1] = float(i);
			vertexPositions[zz+1][2] = 0.0f;
			
			vertexPositions[zz+2][0] = float(j);
			vertexPositions[zz+2][1] = float(i+1);
			vertexPositions[zz+2][2] = 0.0f;
			
			vertexPositions[zz+3][0] = float(j+1);
			vertexPositions[zz+3][1] = float(i+1);
			vertexPositions[zz+3][2] = 0.0f;
			
			
			//cout<<"vertex["<<zz<<"] = ("<<vertexPositions[zz][0]<<", "<<vertexPositions[zz][1]<<", "<<vertexPositions[zz][2]<<"}"<<endl;
			zz += 4;
		}
	}
	//luego crea las caras
	int32_t cara = 0;
	for(int i=0;i<(xmax*ymax);){
		indices[i] = 4;
	}
	//luego las normales
	for(int i=0;i<(xmax*ymax);){
		norms[i][0] = .0f; 
		norms[i][1] = .0f;
		norms[i][2] = .5f;
	}
	//por ultimo los colores
	int ii = 0;
	for(int i = 0; i<(xmax*ymax)*5;++i){
		if(i%1==0){
			colors[i][0]= 1.0f;
			colors[i][1]= .0f; 
			colors[i][2]= .0f;
		}
		else if(i%2==0){
			colors[i][0]= .0f;
			colors[i][1]= 1.0f; 
			colors[i][2]= .0f*i;
		}
		else{
			colors[i][0]= .0f;
			colors[i][1]= .0f; 
			colors[i][2]= 1.0f*i;
		}
		i +=1;
	}
}
// Routine to create a scene graph representing a dodecahedron
SoSeparator * makeStellatedDodecahedron()
{	
	cout<<"makeStellatedDodecahedron"<<endl;
	creaVertexIndicesColores();
	
	SoSeparator *result = new SoSeparator;
	result->ref();
	
	// Define the normals used:
	SoNormal *myNormals = new SoNormal;
	myNormals->vector.setValues(0, faces, norms);
	result->addChild(myNormals);
	SoNormalBinding *myNormalBinding = new SoNormalBinding;
	myNormalBinding->value = SoNormalBinding::PER_FACE;
	result->addChild(myNormalBinding);
	
	// Define material for obelisk
	SoMaterial *myMaterial = new SoMaterial;
	myMaterial->diffuseColor.setValue(.4f, .4f, .4f);
	result->addChild(myMaterial);

	/*
	// Define colors for the faces
	SoMaterial *myMaterials = new SoMaterial;
	myMaterials->diffuseColor.setValues(0, faces, colors);
	result->addChild(myMaterials);
	SoMaterialBinding *myMaterialBinding = new SoMaterialBinding;
	myMaterialBinding->value = SoMaterialBinding::PER_FACE;
	result->addChild(myMaterialBinding);
*/

	// Define coordinates for vertices
	SoCoordinate3 *myCoords = new SoCoordinate3;
	myCoords->point.setValues(0, vertex, vertexPositions);
	result->addChild(myCoords);

	// Define the FaceSet
	SoFaceSet *myFaceSet = new SoFaceSet;
	myFaceSet->numVertices.setValues(0, faces, indices);
	result->addChild(myFaceSet);
	
	
	result->unrefNoDelete();
	return result;
}
int main(int argc, char **argv)
{
	cout<<"main"<<endl;
	// Initialize Inventor and Xt
	HWND myWindow  = SoWin::init(argc, argv, argv[0]);
	if (myWindow == NULL) 
		exit(1);

	SoSeparator *root = makeStellatedDodecahedron();
	root->ref();

	
	
	SoWinExaminerViewer *myViewer = new SoWinExaminerViewer(myWindow);
	myViewer->setSceneGraph(root);
	myViewer->setTitle("Lights");
	//myViewer->setHeadlight(TRUE);
	myViewer->show();

	SoWin::show(myWindow);
	SoWin::mainLoop();
}