#include <Inventor/SoDB.h>
#include <Inventor/Win/SoWin.h>
#include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>

// Positions of all of the vertices:
static float vertexPositions[9][3] =
{
	{ 0.0f,  0.0f,  0.0f},  //0
	{ 0.0f,  1.0f,  0.0f},  //1
	{ 1.0f,  0.0f,  0.0f},  //2
	{ 1.0f,  1.0f,  0.0f},  //3

	{ 2.0f,  0.0f,  0.0f}, //4
	{ 2.0f,  1.0f,  0.0f}, //5
	
	{ 0.0f,  2.0f,  0.0f}, //6
	{ 1.0f,  2.0f,  0.0f}, //7
	
	{ 2.0f,  2.0f,  0.0f}, //8
	
};
 // Connectivity, information; 12 faces with 5 vertices each },
// (plus the end-of-face indicator for each face):

static int32_t indices[20] =
{
	0, 1, 3, 2, SO_END_FACE_INDEX, //1
	2, 4, 5, 3, SO_END_FACE_INDEX, //2
	3, 1, 6, 7, SO_END_FACE_INDEX, //3
	7, 8, 5, 3, SO_END_FACE_INDEX, //4
};
// Colors for the 12 faces
static float colors[4][3] =
{
	{1.0f, .0f, 0.0f}, 
	{1.0f, .0f, 1.0f}, 
	{1.0f, 1.0f, 0.0f}, 
	{.0f, .0f, 1.0f}, 
	
};
// Routine to create a scene graph representing a dodecahedron
SoSeparator * makeStellatedDodecahedron()
{
	SoSeparator *result = new SoSeparator;
	result->ref();

	// Define colors for the faces
	SoMaterial *myMaterials = new SoMaterial;
	myMaterials->diffuseColor.setValues(0, 4, colors);
	result->addChild(myMaterials);
	SoMaterialBinding *myMaterialBinding = new SoMaterialBinding;
	myMaterialBinding->value = SoMaterialBinding::PER_FACE;
	result->addChild(myMaterialBinding);

	// Define coordinates for vertices

	// Define coordinates for vertices
	SoCoordinate3 *myCoords = new SoCoordinate3;
	myCoords->point.setValues(0, 9, vertexPositions);
	result->addChild(myCoords);

	// Define the IndexedFaceSet, with indices into
	// the vertices:
	SoIndexedFaceSet *myFaceSet = new SoIndexedFaceSet;
	myFaceSet->coordIndex.setValues(0, 20, indices);
	result->addChild(myFaceSet);

	result->unrefNoDelete();
	return result;
}
int main(int argc, char **argv)
{
	// Initialize Inventor and Xt
	HWND myWindow  = SoWin::init(argc, argv, argv[0]);
	if (myWindow == NULL) 
		exit(1);

	SoSeparator *root = makeStellatedDodecahedron();
	root->ref();

	
	
	SoWinExaminerViewer *myViewer = new SoWinExaminerViewer(myWindow);
	myViewer->setSceneGraph(root);
	myViewer->setTitle("Lights");
	//myViewer->setHeadlight(TRUE);
	myViewer->show();

	SoWin::show(myWindow);
	SoWin::mainLoop();
}