#ifndef View_H
#define View_H

#include "valores.h"

#include <Inventor/SoDB.h>
#include <Inventor/Win/SoWin.h>
#include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>


const int  vertex = (xmax+1)*(ymax+1);
const int  faces = (xmax*ymax)*5;

// Positions of all of the vertices:
static float vertexPositions[vertex][3];
static int32_t indices[faces];
// Normals for each polygon:
static float norms[1][3] = {.0f, .0f, 1.0f};
// Colors for the 12 faces
static float colors[states][3]=
{ 
	{.7f,  .0f, .7f},
	{1.0f, .0f, .0f},
	{.0f, .0f, 1.0f},
	{.0f, .7f,  .7f},
	{.0f, 1.0f, .0f},
	{.7f, .7f,  .0f},
	{.7f, .0f,  .7f},
	
};
static SoWinExaminerViewer *myViewer;
static SoIndexedFaceSet *myFaceSet;
static int colorsIndex[faces];
class View : public CThread{

public: 
static void colores(){
	int ii = 0;
	for(int i = 0;i< xmax;++i){
		for(int j = 0;j < ymax;++j){
			colorsIndex[ii] = statesMatrix[i][j];
			ii +=1;
		}
	}
}
static void cambia(){
	colores();
	myFaceSet->materialIndex.setValues(0, faces, colorsIndex);
	myViewer->render();
}

void creaVertexIndicesColores(){
	cout<<"creaVertexIndicesColores"<<endl;
	//primero crea los vertices
	int zz = 0;
	for(int i=0; i < ymax+1;++i){
		for(int j=0; j < xmax+1;++j){
			vertexPositions[zz][0] = float(j);
			vertexPositions[zz][1] = float(i);
			vertexPositions[zz][2] = 0.0f;
			//cout<<"vertex["<<zz<<"] = ("<<vertexPositions[zz][0]<<", "<<vertexPositions[zz][1]<<", "<<vertexPositions[zz][2]<<"}"<<endl;
			zz += 1;
		}
	}
	//luego crea las caras
	int32_t cara = 0;
	int jj = 0;
	int flag = 0;
	for(int i=0;i<(xmax*ymax)*5;){
		indices[i] =int32_t(jj);
		indices[i+1] =int32_t(jj+1);
		indices[i+2] =int32_t(jj+xmax+2);
		indices[i+3] =int32_t(jj+xmax+1);
		indices[i+4] =SO_END_FACE_INDEX;
		//cout<<"faces["<<jj<<"] = ("<<indices[i]<<", "<<indices[i+1]<<", "<<indices[i+2]<<", "<<indices[i+3]<<", "<<indices[i+4]<<"}"<<endl;
		i+=5;
		jj += 1;
		flag += 1;
		if(flag==xmax){
			jj+=1;
			flag = 0;
		}
	}
	colores();
}

// Routine to create a scene graph representing a dodecahedron
SoSeparator * makeStellatedDodecahedron()
{	
	cout<<"makeStellatedDodecahedron"<<endl;
	creaVertexIndicesColores();
	
	SoSeparator *result = new SoSeparator;
	result->ref();
	
	// Define the normals used:
	SoNormal *myNormals = new SoNormal;
	myNormals->vector.setValues(0, 1, norms);
	result->addChild(myNormals);
	SoNormalBinding *myNormalBinding = new SoNormalBinding;
	myNormalBinding->value = SoNormalBinding::OVERALL;
	result->addChild(myNormalBinding);
	
	// Define colors for the faces
	SoMaterial *myMaterials = new SoMaterial;
	myMaterials->diffuseColor.setValues(0, states, colors);
	result->addChild(myMaterials);
	SoMaterialBinding *myMaterialBinding = new SoMaterialBinding;
	myMaterialBinding->value = SoMaterialBinding::PER_FACE_INDEXED;
	result->addChild(myMaterialBinding);
	
	// Define coordinates for vertices
	SoCoordinate3 *myCoords = new SoCoordinate3;
	myCoords->point.setValues(0, vertex, vertexPositions);
	result->addChild(myCoords);
	
	// Define the IndexedFaceSet, with indices into
	// the vertices:
	myFaceSet = new SoIndexedFaceSet;
	myFaceSet->materialIndex.setValues(0, faces, colorsIndex);
	myFaceSet->coordIndex.setValues(0, faces, indices);
	result->addChild(myFaceSet);
	
	//don't delete this node
	result->unrefNoDelete();
	return result;
}



	
	View(){}
	~View(){}
	DWORD Run( LPVOID arg ){
		
	// Initialize Inventor and Xt
	HWND myWindow  = SoWin::init("aloha","view" );
	if (myWindow == NULL) 
		exit(1);

	SoSeparator *root = makeStellatedDodecahedron();
	root->ref();
	
	myViewer = new SoWinExaminerViewer(myWindow);
	myViewer->setSceneGraph(root);
	myViewer->setTitle("Lights");
	//myViewer->setHeadlight(TRUE);
	myViewer->show();
	
	SoWin::show(myWindow);
	SoWin::mainLoop();
	return 0;
	}
};
#endif //View_H