#ifndef View_H
#define View_H

#include "valores.h"

#include <Inventor/SoDB.h>
#include <Inventor/Win/SoWin.h>
#include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoQuadMesh.h>
#include <Inventor/nodes/SoShapeHints.h>

const int  vertex = (xmax)*(ymax)*4;
const static int  faces = (xmax*ymax);
// Colors for the 12 faces
static float colors[faces][3];

static SoMaterial *myMaterials;
static SoWinExaminerViewer *myViewer;
// Normals for each polygon:
static float norms[1][3]; 
// Positions of all of the vertex:
static float vertexPositions[vertex][3];
//faces array
static int32_t numvertexPositions[faces];
class View : public CThread{
	
	static void colores(){
		int ii = 0;
		//y por ultimo los colores
		for(int i = 0; i<xmax;++i){
		for(int j = 0; j<ymax;++j){
			if(statesMatrix[i][j][10] == 0){
				colors[ii][0]= .7f;
				colors[ii][1]= .0f; 
				colors[ii][2]= .7f;
			}
			else if(statesMatrix[i][j][10]  == 1){
				colors[ii][0]= 1.0f;
				colors[ii][1]= .0f; 
				colors[ii][2]= .0f;
			}
			else if(statesMatrix[i][j][10]  == 2){
				colors[ii][0]= .0f;
				colors[ii][1]= .0f; 
				colors[ii][2]= 1.0f;
			}
			else if(statesMatrix[i][j][10]  == 3){
				colors[ii][0]= .0f;
				colors[ii][1]= .7f; 
				colors[ii][2]= .7f;
			}
			else if(statesMatrix[i][j][10]  == 4){
				colors[ii][0]= .0f;
				colors[ii][1]= 1.0f; 
				colors[ii][2]= .0f;
			}
			else if(statesMatrix[i][j][10]  == 5){
				colors[ii][0]= .7f;
				colors[ii][1]= .7f; 
				colors[ii][2]= .0f;
			}
			else if(statesMatrix[i][j][10]  == 6){
				colors[ii][0]= .7f;
				colors[ii][1]= .0f; 
				colors[ii][2]= .7f;
			} 
			/*else if(statesMatrix[i][j][0]  == 7){
				colors[ii][0]= .0f;
				colors[ii][1]= .0f; 
				colors[ii][2]= 1.0f;
			}*/
			ii+=1;
		}}

	}
	void createVertexIndex(){
		//create the vertexPositions
		int zz = 0;
		for(int i=0; i < ymax+1;++i){
			for(int j=0; j < xmax+1;++j){
				vertexPositions[zz][0] = float(j);
				vertexPositions[zz][1] = float(i);
				vertexPositions[zz][2] = 0.0f;
				zz += 1;
			}
		}
		//then create the faces  
		int32_t cara = 0;
		for(int i=0;i<(xmax*ymax);++i){
			numvertexPositions[i] = 4;
		}
		colores();
	}
	// Routine to create a scene graph representing the board.
	SoSeparator * initAll()
{
	createVertexIndex();
	SoSeparator *result= new SoSeparator;
	result->ref();
	
	// A shape hints tells the ordering of polygons. 
	SoShapeHints *myHints = new SoShapeHints;
	myHints->shapeType = SoShapeHints::SOLID;
	result->addChild(myHints);
	
	// Define the normals used:
	norms[0][0] = .0f;
	norms[0][1] =.0f;
	norms[0][2] =1.0f;
	SoNormal *myNormals = new SoNormal;
	myNormals->vector.setValues(0, 1, norms);
	result->addChild(myNormals);
	SoNormalBinding *myNormalBinding = new SoNormalBinding;
	myNormalBinding->value = SoNormalBinding::OVERALL;
	result->addChild(myNormalBinding);
	
	// Define colors for the strips
	myMaterials = new SoMaterial;
	myMaterials->diffuseColor.setValues(0, faces, colors);
	result->addChild(myMaterials);
	SoMaterialBinding *myMaterialBinding = new SoMaterialBinding;
	myMaterialBinding->value = SoMaterialBinding::PER_FACE;
	result->addChild(myMaterialBinding);
	
	// Define coordinates for vertexPositions
	SoCoordinate3 *myCoords = new SoCoordinate3;
	myCoords->point.setValues(0, vertex, vertexPositions);
	result->addChild(myCoords);

	// Define the QuadMesh.
	SoQuadMesh *myQuadMesh = new SoQuadMesh;
	myQuadMesh->verticesPerRow = xmax+1;
		
	myQuadMesh->verticesPerColumn = ymax+1;
	result->addChild(myQuadMesh);

	result->unrefNoDelete();
	return result;
}	

public: 

	static void cambia(){
		colores();
		myMaterials->diffuseColor.setValues(0, faces, colors);
		myViewer->render();
	}	

	View(){}
	~View(){}
	DWORD Run( LPVOID arg ){
		
		// Initialize Inventor and Windows
		HWND myWindow  = SoWin::init("aloha","view" );
		if (myWindow == NULL) 
			exit(1);
	
		//create the geometry 
		SoSeparator *root= initAll();
		root->ref();
	
		//create the viewer
		myViewer= new SoWinExaminerViewer(myWindow);
		myViewer->setSceneGraph(root);
		myViewer->setTitle("GraphWorld");
		myViewer->setHeadlight(TRUE);
		myViewer->show();
		
		//create the main loop
		SoWin::show(myWindow);
		SoWin::mainLoop();
		
			return 0;
	}
};
#endif //View_H