#include <iostream>
using namespace std;

#include <Inventor/SoDB.h>
#include <Inventor/Win/SoWin.h>
#include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoFaceSet.h>


const int xmax = 500, ymax = 500;
const int  vertex = (xmax)*(ymax)*4;
const int  faces = (xmax*ymax);
// Positions of all of the vertices:
static float vertices[vertex][3];
static int32_t numvertices[faces];
// Colors for the 12 faces
static float colors[faces][3];
// Normals for each polygon:
static float norms[faces][3];
void creaVertexIndicesColores(){
	cout<<"creaVertexIndicesColores"<<endl;
	//primero crea los  4 vertices
	int zz = 0;
	for(int i=0; i < ymax;++i){
		for(int j=0; j < xmax;++j){
			
			vertices[zz][0] = float(j);
			vertices[zz][1] = float(i);
			vertices[zz][2] = 0.0f;
			
			vertices[zz+1][0] = float(j+1);
			vertices[zz+1][1] = float(i);
			vertices[zz+1][2] = 0.0f;
			
			vertices[zz+2][0] = float(j+1);
			vertices[zz+2][1] = float(i+1);
			vertices[zz+2][2] = 0.0f;
			
			vertices[zz+3][0] = float(j);
			vertices[zz+3][1] = float(i+1);
			vertices[zz+3][2] = 0.0f;
			//cout<<"vertex["<<zz<<"] = ("<<vertexPositions[zz][0]<<", "<<vertexPositions[zz][1]<<", "<<vertexPositions[zz][2]<<"}"<<endl;
			zz += 4;
		}
	}
	//luego crea las caras
	int32_t cara = 0;
	for(int i=0;i<(xmax*ymax);++i){
		numvertices[i] = 4;
	}
	//luego las normales
	for(int i=0;i<(xmax*ymax);++i){
		norms[i][0] = 0.0f;
		norms[i][1] = 0.0f;
		norms[i][2] = 1.0f;
	}
	//y por ultimo los colores
	int jk = 0;
	for(int i = 0; i<faces;++i){
		
		if(jk==0){
			colors[i][0]= .7f;
			colors[i][1]= .0f; 
			colors[i][2]= .7f;
		}
		else if(jk==1){
			colors[i][0]= 1.0f;
			colors[i][1]= .0f; 
			colors[i][2]= .0f;
		}
		else if(jk==2){
			colors[i][0]= .0f;
			colors[i][1]= .0f; 
			colors[i][2]= 1.0f;
		}
		else if(jk==3){
			colors[i][0]= .0f;
			colors[i][1]= .7f; 
			colors[i][2]= .7f;
		}
		else if(jk==4){
			colors[i][0]= .0f;
			colors[i][1]= 1.0f; 
			colors[i][2]= .0f;
		}
		else if(jk==5){
			colors[i][0]= .7f;
			colors[i][1]= .7f; 
			colors[i][2]= .0f;
		}
		else if(jk==6){
			colors[i][0]= .7f;
			colors[i][1]= .0f; 
			colors[i][2]= .7f;
		} 
		else if(jk==7){
			colors[i][0]= .0f;
			colors[i][1]= .0f; 
			colors[i][2]= 1.0f;
		}
		else if(jk==8){
			colors[i][0]= .0f;
			colors[i][1]= .0f; 
			colors[i][2]= 1.0f;
		}
		else if(jk==9){
			colors[i][0]= .7f;
			colors[i][1]= .0f; 
			colors[i][2]= .7f;
			jk = -1;
		}
		jk +=1;
	}
}
SoSeparator * makeObeliskFaceSet()
{
	cout<<"makeStellatedDodecahedron"<<endl;
	creaVertexIndicesColores();
	SoSeparator *automaton = new SoSeparator();
	automaton->ref();

	// Define colors for the faces
	SoMaterial *myMaterials = new SoMaterial;
	myMaterials->diffuseColor.setValues(0, faces, colors);
	automaton->addChild(myMaterials);
	SoMaterialBinding *myMaterialBinding = new SoMaterialBinding;
	myMaterialBinding->value = SoMaterialBinding::PER_FACE;
	automaton->addChild(myMaterialBinding);

	// Define the normals used:
	SoNormal *myNormals = new SoNormal;
	myNormals->vector.setValues(0, faces, norms);
	automaton->addChild(myNormals);
	SoNormalBinding *myNormalBinding = new SoNormalBinding;
	myNormalBinding->value = SoNormalBinding::PER_FACE;
	
	automaton->addChild(myNormalBinding);

	// Define coordinates for vertices
	SoCoordinate3 *myCoords = new SoCoordinate3;
	myCoords->point.setValues(0, vertex, vertices);
	automaton->addChild(myCoords);

	// Define the FaceSet
	SoFaceSet *myFaceSet = new SoFaceSet;
	myFaceSet->numVertices.setValues(0, faces, numvertices);
	automaton->addChild(myFaceSet);

	automaton->unrefNoDelete();
	return automaton;
}
int main(int argc, char **argv)
{
	// Initialize Inventor and Xt
	HWND myWindow  = SoWin::init(argc, argv, argv[0]);
	if (myWindow == NULL) 
		exit(1);

	SoSeparator *root = makeObeliskFaceSet();
	root->ref();
		
	SoWinExaminerViewer *myViewer = new SoWinExaminerViewer(myWindow);
	myViewer->setSceneGraph(root);
	myViewer->setTitle("Lights");
	myViewer->setHeadlight(TRUE);
	myViewer->show();

	SoWin::show(myWindow);
	SoWin::mainLoop();
}